--

with Sequential_IO;

with System;

with Bitmap_Format;
with Bitmap_Routines;

package Bitmap_Handling is
   use Bitmap_Format;
   use Bitmap_Routines;

   -- -- --

   Rectangle : constant Rectangele_Type := (3, 1);

   Nb_Of_Colour_Line_Bytes : constant Length_Type := Get_Nb_Of_Row_Colour_Bytes (Rectangle);

   Nb_Of_All_Colour_Bytes : constant Length_Type := Get_Nb_Of_All_Colour_Bytes (Rectangle);

   Nb_Of_All_Bytes : constant Length_Type := Get_Nb_Of_All_Bytes (Rectangle);

   type Colour_Line_Array_Type is array (1 .. Nb_Of_Colour_Line_Bytes) of Byte_Type;
   for Colour_Line_Array_Type'Component_Size use 8;

   type Colour_Data_Array_Type is array (1 .. Rectangle.Height) of Colour_Line_Array_Type;

   -- -- --

   type Colour_Data_Record_Type is record
      Colour_Data_Array : Colour_Data_Array_Type;
   end record;

   -- -- --

   subtype My_Colour_Pixel_Row_Array_Type is Colour_Pixel_Array_Type (1 .. 3);

   -- -- --

   type Bitmap_Data_Type is record
      Bitmap_Header : Bitmap_Header_Type;
      Colour_Data   : Colour_Data_Record_Type;
   end record;

   for Bitmap_Data_Type use record
      Bitmap_Header at 0 range 0 .. 431;
      --  Colour_Data at 54 range 0 .. 95;
   end record;

   -- -- --

   type Bitmap_Byte_Array_Type is array (1 .. Nb_Of_All_Bytes) of Byte_Type;

   Bitmap_Byte_Array : Bitmap_Byte_Array_Type;

   -- -- --

   package Byte_Io is new Sequential_IO (Element_Type => Byte_Type);
   use Byte_Io;

   procedure Read_Bitmap
     (Ba        : in out Bitmap_Byte_Array_Type;
      File_Name : in     String;
      Nb_Bytes  :    out Length_Type);

   -- -- --

   function Get_Bf_Type_Data (Data_Adress : System.Address) return Unsigned_16_Bit_Integer;
   function Get_Bf_Size_Data (Data_Adress : System.Address) return Unsigned_32_Bit_Integer;
   function Get_Width_Height_Data (Data_Adress : System.Address) return Rectangele_Type;

   procedure Dump_Colour_Data (Data_Adress : System.Address; Count : Positive);

   function Get_Pixel_Row_Array
     (Data_Adress : System.Address) return My_Colour_Pixel_Row_Array_Type;

end Bitmap_Handling;
