--
with Dout.Data_Object;

with System.Storage_Elements;
with System.Address_To_Access_Conversions;

package body Bitmap_Handling_800 is
   use Dout.Data_Object;
   use type System.Storage_Elements.Storage_Offset;

   -- -- --

   procedure Read_Bitmap
     (Ba        : in out Bitmap_Byte_Array_Type;
      File_Name : in     String;
      Nb_Bytes  :    out Length_Type) is

      Input_File : Byte_Io.File_Type;

      Ix : Int32_T := 0;
   begin
      Open (Input_File, In_File, File_Name);

      while not End_Of_File (Input_File) loop
         Ix := Ix + 1;

         Byte_Io.Read (File => Input_File, Item => Ba (Ix));
      end loop;

      Close (Input_File);

      Nb_Bytes := Ix;
   exception
      when others =>
         Close (Input_File);
         Nb_Bytes := Ix;

         raise;
   end Read_Bitmap;

   -- -- --

   procedure Write_Bitmap (Ba : in out Bitmap_Byte_Array_Type; File_Name : in String) is

      Output_File : Byte_Io.File_Type;

   begin
      -- -- --
      declare
      begin
         Open (Output_File, Out_File, File_Name);

      exception
         when others =>
            Create (Output_File, Out_File, File_Name);
      end; -- declare

      for Ix in Ba'Range loop

         Byte_Io.Write (File => Output_File, Item => Ba (Ix));
      end loop;

      Close (Output_File);

   exception
      when others =>
         Close (Output_File);

         raise;
   end Write_Bitmap;

   -- -- --

   function Get_Bf_Type_Data (Data_Adress : System.Address) return Unsigned_16_Bit_Integer is
      Bf_Type : Unsigned_16_Bit_Integer;
      for Bf_Type'Address use Data_Adress;
   begin
      return Bf_Type;
   end Get_Bf_Type_Data;

   -- -- --

   function Get_Bf_Size_Data (Data_Adress : System.Address) return Unsigned_32_Bit_Integer is
      Bitmap_Data : Bitmap_Data_Type;
      for Bitmap_Data'Address use Data_Adress;
   begin

      return Bitmap_Data.Bitmap_Header.Bf_Size;
   end Get_Bf_Size_Data;

   -- -- --

   function Get_Width_Height_Data (Data_Adress : System.Address) return Rectangele_Type is
      Bitmap_Data : Bitmap_Data_Type;
      for Bitmap_Data'Address use Data_Adress;
   begin

      return (Bitmap_Data.Bitmap_Header.Bi_Width, Bitmap_Data.Bitmap_Header.Bi_Height);
   end Get_Width_Height_Data;

   -- -- --

   procedure Dump_Colour_Data
     (Data_Adress : System.Address;
      Row_Index   : Height_Index_Type;
      Count       : Positive) is

      Bitmap_Data : Bitmap_Data_Type;
      --
      for Bitmap_Data'Address use Data_Adress;
   begin
      Dump (Bitmap_Data.Colour_Data.Colour_Data_Array (Row_Index)'Address, Count);
   end Dump_Colour_Data;

   -- -- --
   -- -- --

   function Get_Pixel_Row_Array
     (Data_Adress : System.Address;
      Row_Index   : Height_Index_Type) return Colour_Pixel_Row_Array_Type is

      Bitmap_Data : Bitmap_Data_Type;
      --
      for Bitmap_Data'Address use Data_Adress;

      Colour_Line_Array : Colour_Line_Array_Type;

      Colour_Pixel_Row_Array : Colour_Pixel_Row_Array_Type;
      --
      for Colour_Pixel_Row_Array'Address use Colour_Line_Array'Address;
   begin
      Colour_Line_Array := Bitmap_Data.Colour_Data.Colour_Data_Array (Row_Index);

      return Colour_Pixel_Row_Array;
   end Get_Pixel_Row_Array;

   -- -- --

   package Row_Colour_Data_Array_Address_Conversions is new System.Address_To_Access_Conversions
     (Colour_Pixel_Row_Array_Type);

   procedure Set_Pixel_Row_Array
     (Data_Adress            : in     System.Address;
      Row_Index              : in     Height_Index_Type;
      Colour_Pixel_Row_Array : in out Colour_Pixel_Row_Array_Type) is

      Bitmap_Data : Bitmap_Data_Type;
      --
      for Bitmap_Data'Address use Data_Adress;

      Row_Colour_Data_Array_Address : System.Address;

      Colour_Pixel_Row_Array_Access : Row_Colour_Data_Array_Address_Conversions.Object_Pointer;
   begin

      Row_Colour_Data_Array_Address :=
        Bitmap_Data.Colour_Data.Colour_Data_Array (Row_Index)'Address;

      Colour_Pixel_Row_Array_Access :=
        Row_Colour_Data_Array_Address_Conversions.To_Pointer (Row_Colour_Data_Array_Address);

      Colour_Pixel_Row_Array_Access.all := Colour_Pixel_Row_Array;

   end Set_Pixel_Row_Array;

   -- -- --

end Bitmap_Handling_800;
