--

with Sequential_IO;

with System;

with Bitmap_Format;
with Bitmap_Routines;

package Bitmap_Handling_800 is
   use Bitmap_Format;
   use Bitmap_Routines;

   -- -- --

   Rectangle : constant Rectangele_Type := (800, 800);

   Nb_Of_Colour_Line_Bytes : constant Length_Type := Get_Nb_Of_Row_Colour_Bytes (Rectangle);

   Nb_Of_All_Colour_Bytes : constant Length_Type := Get_Nb_Of_All_Colour_Bytes (Rectangle);

   Nb_Of_All_Bytes : constant Length_Type := Get_Nb_Of_All_Bytes (Rectangle);

   type Colour_Line_Array_Type is array (1 .. Nb_Of_Colour_Line_Bytes) of Byte_Type;
   for Colour_Line_Array_Type'Component_Size use 8;

   subtype Height_Index_Type is Index_Type range 1 .. Rectangle.Height;

   type Colour_Data_Array_Type is array (Height_Index_Type) of Colour_Line_Array_Type;

   -- -- --

   type Colour_Data_Record_Type is record
      Colour_Data_Array : Colour_Data_Array_Type;
   end record;

   -- -- --

   subtype Width_Index_Type is Index_Type range 1 .. Rectangle.Width;

   subtype Colour_Pixel_Row_Array_Type is Colour_Pixel_Array_Type (Width_Index_Type);

   -- -- --

   type Bitmap_Data_Type is record
      Bitmap_Header : Bitmap_Header_Type;
      Colour_Data   : Colour_Data_Record_Type;
   end record;

   for Bitmap_Data_Type use record
      Bitmap_Header at 0 range 0 .. 431;
      --
   end record;

   -- -- --

   type Bitmap_Byte_Array_Type is array (1 .. Nb_Of_All_Bytes) of Byte_Type;

   Bitmap_Byte_Array : Bitmap_Byte_Array_Type;

   -- -- --

   package Byte_Io is new Sequential_IO (Element_Type => Byte_Type);
   use Byte_Io;

   procedure Read_Bitmap
     (Ba        : in out Bitmap_Byte_Array_Type;
      File_Name : in     String;
      Nb_Bytes  :    out Length_Type);

   procedure Write_Bitmap (Ba : in out Bitmap_Byte_Array_Type; File_Name : in String);

   -- -- --

   procedure Dump_Colour_Data
     (Data_Adress : System.Address;
      Row_Index   : Height_Index_Type;
      Count       : Positive);

   -- -- --

   function Get_Bf_Type_Data (Data_Adress : System.Address) return Unsigned_16_Bit_Integer;
   function Get_Bf_Size_Data (Data_Adress : System.Address) return Unsigned_32_Bit_Integer;
   function Get_Width_Height_Data (Data_Adress : System.Address) return Rectangele_Type;

   function Get_Pixel_Row_Array
     (Data_Adress : System.Address;
      Row_Index   : Height_Index_Type) return Colour_Pixel_Row_Array_Type;

   -- -- --

   procedure Set_Pixel_Row_Array
     (Data_Adress            : in     System.Address;
      Row_Index              : in     Height_Index_Type;
      Colour_Pixel_Row_Array : in out Colour_Pixel_Row_Array_Type);

   -- -- --

end Bitmap_Handling_800;
