--

with Ada.Task_Attributes;

package body Completion_Pool_Executors is

   overriding procedure Submit
     (Exec     : in out Completion_Pool_Executor_Type;
      Callable : in     My_Callables.Any_Callable_Access_Type;
      Params   : in     My_Executors.Parameter_Type;
      Future   : in     My_Futures.Any_Future_Access_Type) is
      --
      Worker_Id : Positive := Positive'First;
   begin

      Exec.Controller.Get (Worker_Id);

      Exec.Pool (Worker_Id).Fork (Callable, Params, Future);
   end Submit;

   -- -- --

   overriding procedure Take
     (Exec   : in out Completion_Pool_Executor_Type;
      Future :    out My_Futures.Any_Future_Access_Type) is
      --
      Tmp_Future : My_Futures.Any_Future_Access_Type;
   begin
      Exec.Future_Buffer.Get (Tmp_Future);
      --
      -- Wait for the next result

      Future := Tmp_Future;
   end Take;

   -- -- --
   -- -- --

   package My_Task_Attributes is new Ada.Task_Attributes
     (Attribute     => Any_Completion_Pool_Executor_Access_Type,
      Initial_Value => null);

   procedure Initialize (Excutor : in out Completion_Pool_Executor_Type) is
   begin

      for Ix in Excutor.Pool'Range loop
         My_Task_Attributes.Set_Value
           (Val => Excutor'Unchecked_Access,
            T   => Excutor.Pool (Ix)'Identity);
      end loop;

   end Initialize;

   -- -- --

   function Set_Cpu return System.Multiprocessors.CPU_Range is
      use System.Multiprocessors;
   begin
      Task_Nb := Integer'Succ (Task_Nb);

      return CPU_Range (Task_Nb mod My_Number_Of_Cpus + 1);

   exception
      when Constraint_Error =>
         Task_Nb := 0;

         return 1;
   end Set_Cpu;

   -- -- --
   -- -- --

   task body Worker_Task_Type is separate;

end Completion_Pool_Executors;
